/**
 * Simple example to show how to place an object in 3D space.
 * Also demonstrates how to position a pointLight and how to overlay 2D graphics over the 3D space.
 **/

import processing.opengl.*;
import javax.media.opengl.*; 

int NUM_DAYS = 40;
int COLS = 9;
int ROWS = 9;
int CURRENT_DAY = 0;
long changeLength = 1000L;

long timeToChange = changeLength;
float lightX = 20f;
float lightY = 20f;
float lightZ = 20f;
float rotObjX = 75f;
float rotObjY = 0f;
float rotObjZ = 45f;
float rotCamX = 0f;
float rotCamY = 0f;
float rotCamZ = 0f;
float camX = 0f;
float camY = 0f;
float camZ = 300f;
boolean isChanging = true;

Cell[][] cells = new Cell[NUM_DAYS][COLS * ROWS];
Cell[] useCells = new Cell[COLS * ROWS]; // = cells[0];

void setup() 
{
  size(800, 600, OPENGL); 
  //frame.setResizable(true);
  background(0); 
  //stroke(255);
  lightX = width/2f + 0;
  lightY = height/2f + 0;
  lightZ = 400f;
  textFont(loadFont("text.vlw"), 24);

  initializeCells();
}

void initializeCells()
{
  for (int dday = 0; dday < NUM_DAYS; dday++)
  {
    for (int col = 0; col < COLS; col++)
    {
      for (int row = 0; row < ROWS; row++)
      {
        cells[dday][col * COLS + row] = new Cell(dday, col, row, 
        random(-250f, 250f), 
        color(random(110,155), random(110,155), random(110,155)));
      }
    }
  }

  //init changing cells to be the same as the first day...
  for (int i = 0; i < cells[0].length; i++)
  {
    useCells[i] = new Cell(cells[0][i].dday, cells[0][i].col, cells[0][i].row, cells[0][i].h, cells[0][i].clr);
  } 
}

void draw()
{
  background(0);

  /** 
   * Setup Camera :
   * convenience method that initializes PROJECTION matrix & MODELVIEW matrix
   **/
  beginCamera();
  {
    //position camera at default position

    camera(); 

    //move camera
    translate(camX, camY, camZ);

    //rotate camera in place
    rotateX(radians(rotCamX));
    rotateY(radians(rotCamY));
    rotateZ(radians(rotCamZ));
  }
  endCamera();

  /**
   * Draw objects in 3D space
   **/

  //draw point representing lightbulb
    noLights();
    pushMatrix();
   {
      translate(lightX, lightY, lightZ);
      sphere(10); //0,0,10,10);
      text("light!", 10, 0);
    }
    popMatrix();

  pointLight(255,255,255, lightX, lightY, lightZ); 
  //ambientLight(255,255,255);

  pushMatrix();
  {
    //rotate object around center  
    translate(width/2, height/2, 0);
    rotateX((float)Math.toRadians(rotObjX));
    rotateY((float)Math.toRadians(rotObjY));
    rotateZ((float)Math.toRadians(rotObjZ));
    translate(-width/2, -height/2, 0);

    for (int i = 0; i < ROWS * COLS; i++)
    {
      Cell c = useCells[i];
      c.update();
      c.draw( (width - height) / 2, 10, (height - 20) / ROWS, (height - 20) / COLS); 
    }
  }
  popMatrix();

  /**
   * Now lets draw some 2D things over our 3D world.
   * These will *not* be affected by our 3D transformations.
   **/

  

  /**
   * Check if it's time to change to a new day of data
   **/
  if (isChanging == true && millis() > timeToChange)
  {
    changeDay();
    timeToChange += changeLength;  
  }
  
  //reset camera 
  camera();
  noLights();
  fill(255);
  text("fps = " + frameRate, 10, height - 5);
}


void changeDay()
{
  long startTime = millis() + 250;

  int NEXT_DAY = (CURRENT_DAY + 1) % (NUM_DAYS);

  for (int i = 0; i < ROWS * COLS; i++)
  {
    long endTime = millis() + (long)random(500,2000);

    Cell uc = useCells[i];
    Cell nc = cells[NEXT_DAY][i];

    new MovementTimer(uc, startTime, endTime, nc.h - uc.h, 
    red(nc.clr) - red(uc.clr), green(nc.clr) - green(uc.clr), blue(nc.clr) - blue(uc.clr));     
  }

  CURRENT_DAY = NEXT_DAY;
}

void keyPressed()
{
  switch(key)
  {
    //translate lights
  case 'a':
    lightX += 5;
    break;
  case 'z':
    lightX -= 5;
    break;
  case 's':
    lightY -= 5;
    break;
  case 'x':
    lightY += 5;
    break;
  case 'd':
    lightZ += 5;
    break;
  case 'c':
    lightZ -= 5;
    break;

    //rotate object
  case '1':
    rotObjX -= 1f;
    break;
  case 'q':
    rotObjX += 1f;
    break;
  case '2':
    rotObjY -= 1f;
    break;
  case 'w':
    rotObjY += 1f;
    break;
  case '3':
    rotObjZ -= 1f;
    break;
  case 'e':
    rotObjZ += 1f;
    break;

    //rotate camera
  case '8':
    rotCamX -= 1f;
    break;
  case 'i':
    rotCamX += 1f;
    break;
  case '9':
    rotCamY -= 1f;
    break;
  case 'o':
    rotCamY += 1f;
    break;
  case '0':
    rotCamZ -= 1f;
    break;
  case 'p':
    rotCamZ += 1f;
    break;


    //translate camera
  case 'h':
    camX -= 1f;
    break;
  case 'n':
    camX += 1f;
    break;
  case 'j':
    camY -= 1f;
    break;
  case 'm':
    camY += 1f;
    break;
  case 'k':
    camZ -= 1f;
    break;
  case ',':
    camZ += 1f;
    break;

    //reset
  case ' ':
    reset();
    break;

    //change days 
  case 'g':
    isChanging = !isChanging;
    break;  
  }
}


void mouseDragged() 
{
  float rate = 1f;
  rotObjX += (pmouseY-mouseY) * rate;
  rotObjZ += (mouseX-pmouseX) * rate;
}

void reset()
{ 
  lightX = width/2f + 30f;
  lightY = height/2f + 30f;
  lightZ = 400f;
  rotObjX = 0f;
  rotObjY = -30f;
  rotObjZ = 0f;
  rotCamX = 0f;
  rotCamY = 0f;
  rotCamZ = 0f;
  camX = 0f;
  camY = 0f;
  camZ = 0f;

}







